.srfimBasecall <- function(object, info, ...,
                              sample.name="",
                              filename="",
                              filter=logical(0),
                              trim=integer(0),
                              score=c("Entropy", "Srfim","LogRatio2","LogRatio1","Phred","None"))
  {
    if (!require(ShortRead, quietly=TRUE)) {
      stop("Package ShortRead required to do base calls")
    }

    if (missing(sample.name))
      sample.name <- "seraph"
    
    ids <- BStringSet(do.call(paste,c(sample.name,pData(info),sep="_")))

    probs <- object@probs
    ncycles <- dim(probs)[3]
    if ( !missing(trim) && length(trim) > 0) {
      if ((min(trim) < 1) || (max(trim)>ncycles)) {
        stop("trim is out of range")
      }

      probs <- probs[,,trim]
    }

    if ( !missing(filter) && length(filter) > 0) {
      if (length(filter) != dim(probs)[1])
        stop("filter is too short, should be same length and probs")
    } else {
      filter <- !object@junk.reads
    }
    probs <- probs[filter,,]
    ids <- ids[filter]
    
    nreads <- dim(probs)[1]
    ncycles <- dim(probs)[3]

    type <- 'n'
    score <- match.arg(score)
    score <- switch(score,Phred='p',LogRatio1='l',LogRatio2='m',Srfim='s',Entropy='e',None='n')
    
    if (score != 's') {
      res <- .Call("callBases",probs,type,score,PACKAGE="srfim")
      calls <- DNAStringSet(res[[1]],start=1,width=ncycles)

      if (score != 'n') {
        qual.maker <- switch(score,
                             l=SFastqQuality,
                             m=SFastqQuality,
                             FastqQuality)
        
        quals <- qual.maker(BStringSet(res[[2]],start=1,width=ncycles))
      } else {
        quals <- FastqQuality(BStringSet(Views(paste(rep("I",ncycles),collapse=""), start=rep(1,nreads), width=ncycles)))
      }

      res <- new("ShortReadQ",sread=calls,quality=quals,id=ids)
    } else {
      res <- .Call("callProbs", probs, type, PACKAGE="srfim")
      calls <- DNAStringSet(res[[1]],start=1,width=ncycles);

      quals <- SFastqQuality(BStringSet(res[[2]],start=1,width=ncycles));

      nalt <- BStringSet(res[[3]],start=1,width=ncycles);
      alt.calls <- DNAStringSet(res[[4]])
      alt.scores <- SFastqQuality(BStringSet(res[[5]]));

      res <- new("FuzzyShortReadQ", sread=calls, quality=quals, id=ids,
                 nalt=nalt, altread=alt.calls, altqual=alt.scores)
    }

    if (!missing(filename) && length(filename) > 0) {
      srfimWrite(res, filename)
    }
    
    return(res)
  }

.writeFuzzyFastq <- function(object, file="", ...)
  {
    if (length(file)>0) {
      if (grepl("ffq$", file)) {
        seqfile <- gsub("\\.ffq$", "_seq.ffq", file)
        altseqfile <- gsub("\\.ffq$", "_altseq.ffq", file)
      } else {
        seqfile <- paste(file, "_seq.ffq", sep="")
        altseqfile <- paste(file, "_altseq.ffq", sep="")
      }
    } else {
      seqfile <- file
      altseqfile <- file
    }

    tmpids <- BStringSet(paste(id(object), width(object@altread), sep=":"))
    tmp <- new("ShortReadQ", sread=sread(object), id=tmpids, quality=quality(object))
    writeFastq(tmp, file=seqfile)

    keep <- width(object@altread)>0
    tmpids <- BStringSet(paste(id(object), object@nalt, sep=":"))
    tmp <- new("ShortReadQ", sread=object@altread[keep], id=tmpids[keep], quality=object@altqual[keep])
    writeFastq(tmp, file=altseqfile)
  }
