###########
# SrfimPreprocessResult
setClass("SrfimPreprocessResult",
         representation=representation(intensity="array",
           M="array",
           P="list"),
         prototype=prototype(array(0,c(0,0,0)),
           M=array(0,c(0,0,0)),
           P=vector("list",0)))

setMethod("initialize", "SrfimPreprocessResult",
          function(.Object, intensity, M, P) {
            .Object@intensity <- intensity
            .Object@M <- M
            .Object@P <- P
            .Object
          })

######
# SrfimModelResult
setClass("SrfimModelResult",
         representation=representation(probs="array",
           line.pars="matrix",
           mu="list",
           block.factor="integer",
           junk.reads="logical",
           slog.x="array",
           shift="numeric",
           metrics="environment"),
         prototype=prototype(
           probs=array(0,c(0,0,0)),
           line.pars=matrix(0,nr=0,nc=0),
           mu=vector("list", 0),
           block.factor=integer(0),
           junk.reads=logical(0),
           slog.x=array(0,c(0,0,0)),
           shift=numeric(0),
           metrics=new.env()))

setMethod("initialize", "SrfimModelResult",
          function(.Object, probs, line.pars, mu,
                   block.factor, junk.reads,
                   slog.x, shift)
          {
            .Object@probs <- probs
            .Object@line.pars <- line.pars
            .Object@mu <- mu
            .Object@block.factor <- block.factor
            .Object@junk.reads <- junk.reads
            .Object@slog.x <- slog.x
            .Object@shift <- shift
            .Object@metrics <- new.env()
            .Object
          })

#############
# FuzzyShortReadQ
setClass("FuzzyShortReadQ",
         contains="ShortReadQ",
         representation=representation(
           nalt="BStringSet",
           altread="DNAStringSet",
           altqual="QualityScore"))

######
# SrfimQAResult
setClass("SrfimQAResult",
         representation=representation(
           line.pars="matrix",
           mu="list",
           junk.reads="logical",
           nblocks="integer",
           SNR="numeric",
           ENT="numeric",
           SNR2="numeric"),
         prototype=prototype(
           line.pars=matrix(0,nr=0,nc=0),
           mu=vector("list", 0),
           junk.reads=logical(0),
           nblocks=integer(0),
           SNR=numeric(0),
           ENT=numeric(0),
           SNR2=numeric(0)))

setMethod("initialize", "SrfimQAResult",
          function(.Object, line.pars,
                   mu, junk.reads, nblocks,
                   SNR, ENT, SNR2) {
            .Object@line.pars <- line.pars
            .Object@mu <- mu
            .Object@junk.reads <- junk.reads
            .Object@nblocks <- nblocks
            .Object@SNR <- SNR
            .Object@ENT <- ENT
            .Object@SNR2 <- SNR2
            .Object
          })
#####
# SrfimResult
#
# Result of running srfim
setClass("SrfimResult",
         representation(basecalls="ShortReadQ",
                        qa="SrfimQAResult",
                        filter.type="character",
                        filter="logical",
                        filename="character",
                        call="call"))

#####
# SrfimBatch
#
# Container for a batch of Solexa data on which
# srfim basecaller/qa can be performed
setClass("SrfimBatch",
         representation(path="character",
                        type="character",
                        batchName="character"))

setMethod("initialize", "SrfimBatch",
          function(.Object, path, type, batchName) {
            .Object@path <- path
            .Object@type <- type
            .Object@batchName <- batchName
            .Object
          })


