.plot.intensities <- function(x,cycle=1,calls=NULL,...)
  {
    if (length(dim(x)) == 2) {
      .plot.one.cycle(x,...)
    } else if ((missing(cycle) | is.null(cycle))) {
      calls <- .get.calls(calls)
      .plot.one.cycle(x[,,1],calls=calls,...)
    } else {
      stopifnot(cycle <= dim(x)[3])
      calls <- .get.calls(calls)
      .plot.one.cycle(x[,,cycle],calls=calls,...)
    }
  }

.get.calls <- function(calls=NULL,cycle=1)
  {
    if(is.null(calls))
      return()

    match(as.character(Biostrings::DNAStringSet(calls,start=cycle,width=1)),Biostrings::DNA_ALPHABET)
  }

.plot.data <- function(x,cycle=1,...)
  {
    if (length(dim(x)) == 2) {
      .plot.one.cycle(x,...)
    } else if (!(missing(cycle) | is.null(cycle))) {
      .plot.one.cycle(x[,,1],...)
    } else {
      stopifnot(cycle <= dim(x)[3])
      .plot.one.cycle(x[,,cycle],...)
    }
  }

.plot.one.cycle <- function(x,sample.size=10000,call.col=c("blue","purple","green","red","yellow"),calls=NULL,lims=NULL,plot.diag=TRUE,...)
  {
    if(!is.null(call.col) & !is.null(calls))
      call.col <- call.col[calls]
    
    if (is.null(lims)) {
      lims <- apply(x,2,quantile,p=c(.1,.99),names=FALSE)
    } else {
      lims <- matrix(rep(lims,4),nr=2)
    }

#    keep <- which(apply(sapply(1:ncol(x),
#                               function(i) x[,i] >= min(lims[,i]) & x[,i] <= max(lims[,i])),
#                        1,all))
    keep <- 1:nrow(x)
    sample.size <- pmin(sample.size,length(keep))
    keep <- sample(keep,sample.size)

    lims <- c(min(lims[1,]),max(lims[2,]))
    call.col <- if (is.null(calls)) "black" else call.col[keep]

    panel.f <- function(x,y,...) {
      points(x,y,...)
      if (plot.diag)
        abline(a=0,b=1)
    }
    pairs(x[keep,],cex=.1,col=call.col,xlim=lims,ylim=lims,panel=panel.f,...)
  }

.plot.lines <- function(coefs)
  {
    ab21 <- coefs[,1]
    ab12 <- coefs[,2]
    
    abline(coef=ab21)
    abline(a=-ab12["a"]/ab12["b"],b=1/ab12["b"])
  }

.plot.clustering <- function(x,smp.inds,z,parameters,call.colors=c("blue","purple","green","red"),cex=.1,...)
  {
    x <- matrix(x[smp.inds],nc=4,dimnames=dimnames(x)[c(1,2)])
    z <- matrix(z[smp.inds],nc=4,dimnames=dimnames(x)[c(1,2)])
    
    lims <- apply(x,2,quantile,p=c(.1,.99),names=FALSE)
    lims <- c(min(lims[1,]),max(lims[2,]))

    calls <- mclust::map(z)
    rgbmat <- sapply(call.colors,col2rgb)

    col <- rgbmat[,calls]
    weights <- z[cbind(1:nrow(z),calls)]
    col <- sweep(col,2,weights,FUN="*")
    col <- apply(col,2,function(x) rgb(x[1],x[2],x[3],max=255))

    panelf <- function(i,j,...)
      {
        i <- i[1]; j <- j[1]
        points(x[,i],x[,j],...)

        mu <- parameters$mean[c(i,j),]
        sigma <- parameters$variance$sigma[c(i,j),c(i,j),]

        # plot cluster centers
        points(mu[1,],mu[2,],pch=8,cex=1)
        for (k in 1:ncol(mu)) {
          lines(ellipse(sigma[,,k],centre=mu[,k],level=.75))
        }
        
      }

    mat <- matrix(rep(1:4,each=2),nc=4)
    dimnames(mat) <- dimnames(x)
    pairs(mat,panel=panelf,xlim=lims,ylim=lims,col=col,cex=cex,...)
  }

