.srfimQA <- function(model, ...)
  {
    nreads <- length(model@junk.reads)
    junk.reads <- model@junk.reads
    
    line.pars <- model@line.pars[!junk.reads,]
    probs <- model@probs[!junk.reads,,]

    ncycles <- dim(probs)[3]
    SNR <- getSNR(model)
    ENT <- rowSums(getENT(model))
    SNR2 <- getSNR2(model)
    
    new("SrfimQAResult",
        line.pars=model@line.pars,
        mu=model@mu,
        junk.reads=model@junk.reads,
        nblocks=length(unique(model@block.factor)),
        SNR=SNR,
        ENT=ENT,
        SNR2=SNR2)
  }


# SNR is defined as ratio of average squared distance between model lines
# and average of variance
#
# (1/N * ||X(alpha-beta)||^2_2) / (1/2 * (sigma_a^2 + sigma_b^2))
#
compute.SNR <- function(line.pars,ncycles)
  {
    X <- cbind(1,0:(ncycles-1))
    res <- line.pars[,c("alpha_0","alpha_1")]-line.pars[,c("beta_0","beta_1")]
    res <- tcrossprod(X,res)
    res <- colSums(res^2)/ncycles
    res/(0.5 * rowSums(line.pars[,c("sigma_a","sigma_b")]^2))
  }

compute.SNR2 <- function(line.pars, mu, ncycles)
  {
    nreads <- nrow(line.pars)
    mua <- rowMeans(mu$alpha, dims=2)
    mub <- rowMeans(mu$beta, dims=2)
    mu <- mua + mub
    mubar <- colMeans(mu)

    sigma <- 0.5 * rowSums(line.pars[,c("sigma_a", "sigma_b")]^2)
    
    X <- cbind(1, 0:(ncycles-1))
    res1 <- compute.SNR(line.pars, ncycles)

    ab <- line.pars[,c("alpha_0", "alpha_1")] - line.pars[,c("beta_0", "beta_1")]
    res2 <- tcrossprod(X, ab)
    res2 <- colMeans(res2*mubar)/sigma

    res3 <- mean(rowSums(mu^2))/sigma

    res1 + -2 * res2 + res3
  }

# ENT is defined as entropy of z in each read-cycle position
compute.ENT <- function(z)
  {
    -basesum(z * log2(z))
  }



